package com.mgh.pmdm.parques.viewmodel

import android.app.Application
import android.util.Log
import android.view.View
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
//import androidx.lifecycle.ViewModel
import com.mgh.pmdm.parques.model.Park
import com.mgh.pmdm.parques.repository.ParkRepository

// Definimos el ViewModel como AndroidViewModel, para poder acceder al contexto
class MainViewModel(application: Application): AndroidViewModel(application) {

    // Definición de atributos
    private val _currentPark = MutableLiveData<Park>()
    // Parque que se está editando actualmente

    // Proporcionamos acceso público
    val currentPark: LiveData<Park> =_currentPark

    fun setCurrentPark(park: Park){
        Log.d("Debug SetCurrentPark", park.toString())
        _currentPark.value=park
    }


    // Definimos los atributos Livedata para gestionar los clics,
    // de manera que puedan ser obervador por un observer en el fragmento
    val parkLongClicked: MutableLiveData<Park> by lazy {
        MutableLiveData<Park>()
    }
    val parkClicked: MutableLiveData<Park> by lazy {
        MutableLiveData<Park>()
    }

    // LiveData para el adaptador mediante un atrobuto de soporte privado:
    // El adaptador del RecyclerView también será un LiveData para
    // poder ser observable desde la vista.
    private val _adaptador = MutableLiveData<AdaptadorParques>().apply{

        // Mediante value (setValue) establecemos el valor que contendrá el MutableLiveData
        // de tipo AdaptadorParques. Aqui proporcionamos también los callbacks que
        // se deben invocar cuando se produzcan clicks.
        value= AdaptadorParques(
            {park:Park, v: View -> parkClickedManager(park, v)},
            {park: Park, v: View -> ParkLongClickedManager(park, v)},
            getApplication<Application>().applicationContext // Esto lo podemos hacer gracias a extender de AndroidViewModel
        )
    }

    // Definimos la propiedad que podremos leer desde la vista
    // (_adapter es un atributo de soporte provado)
    val adaptador:MutableLiveData<AdaptadorParques> =_adaptador


    // Definimos las funciones de callback para la gestión de eventos
    // El evento lo gestionará el FirstFragment, pero será avisado
    // por el ViewModel cuando se producen los clicks sobre cada element.
    private fun parkClickedManager(park: Park, v: View){
        // Para establecer el valor del liveData utilizamos value
        parkClicked.value=park
    }

    private fun ParkLongClickedManager(park:Park, v: View):Boolean {
        // Para establecer el valor del liveData utilizamos Value
        parkLongClicked.value=park
        return true
    }

    // Método para eliminar un parque de la lista
    // Se modificará el modelo a través del repositorio, y
    // se notificará al Adaptador para que se refresque.

    fun removePark(park:Park): Int{
        val index= ParkRepository.getInstance(getApplication<Application>().applicationContext).removePark(park)

        adaptador.value?.notifyItemRemoved(index)
        return index
    }


    // Método para guardar el parque
    fun savePark(park:Park):Boolean {
        // Comprobamos si currentPark existe o es nulo

        // Si existe, deberemos modificarlo
        val repoinstance=ParkRepository.getInstance(getApplication<Application>().applicationContext)
        if (_currentPark.value != null) repoinstance.update(_currentPark.value as Park, park)
        // Si no, deberemos añadirlo
        else repoinstance.add(park)

        // Actualizamos el CurrentPark con una copia del parque nuevo
        _currentPark.value = park.copy()

        // Finalmente, mostramos un snackbar con la confirmación
        return true
    }
}